local AceAddon = AceLibrary("AceAddon-2.0")
local Tablet = AceLibrary("Tablet-2.0")

local L = AceLibrary("AceLocale-2.2"):new("Cartographer_Trainers")
L:RegisterTranslations("enUS", function() return {
	["Filter"] = true,
	["Filter out trainers"] = true,
	
	["Select all"] = true,
	["Select none"] = true,
    
    ["Trainer"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Filter"] = "필터",
	["Filter out trainers"] = "상급 기술자 아웃 필터",
	
	["Select all"] = "모두 선택",
	["Select none"] = "선택 없음",

        ["Trainer"] = "상급 기술자",
} end)

local mod
if AceLibrary:HasInstance("AceDebug-2.0") then 
    mod = Cartographer:NewModule("Trainers", "AceConsole-2.0", "AceEvent-2.0", "AceDebug-2.0")
    mod:SetDebugging(false)
else
    mod = Cartographer:NewModule("Trainers", "AceConsole-2.0", "AceEvent-2.0")
    -- mod.Debug = function() return end
end

mod.icon = {
	["Trainer"] = {
		text = L["Trainer"],
		path = "Interface\\Icons\\INV_Misc_Book_03",
		width = 12,
		height = 12
	},
}

mod.trainers = {}

function mod:OnInitialize()
	self.db = Cartographer:AcquireDBNamespace("Trainers")
	Cartographer:RegisterDefaults("Trainers", "profile", {
		filter = {
			['*'] = true,
		}
	})

	local aceopts = {}
	aceopts.toggle = {
		name = AceLibrary("AceLocale-2.2"):new("Cartographer")["Enabled"],
		desc = AceLibrary("AceLocale-2.2"):new("Cartographer")["Suspend/resume this module."],
		type  = 'toggle',
		order = -1,
		get   = function() return Cartographer:IsModuleActive(self) end,
		set   = function() Cartographer:ToggleModuleActive(self) end,
	}

    --[[
	aceopts.filter = {
		name = L["Filter"],
		desc = L["Filter out quests"],
		type = 'group',
		args = {
			all = {
				name = L["Select all"],
				desc = L["Select all"],
				type = 'execute',
				func = function()
					for k in pairs(self.icon) do
						self:ToggleShowingQuests(k, true)
					end
				end,
				order = 1,
			},
			none = {
				name = L["Select none"],
				desc = L["Select none"],
				type = 'execute',
				func = function()
					for k in pairs(self.icon) do
						self:ToggleShowingQuests(k, false)
					end
				end,
				order = 2,
			},
			blank = {
				type = 'header',
				order = 3,
			}
		}
	}
	for k,v in pairs(self.icon) do
		local k = k
		aceopts.filter.args[k] = {
			name = v.text,
			desc = v.text,
			type = 'toggle',
			get = function()
				return self:IsShowingQuests(k)
			end,
			set = function(value)
				return self:ToggleShowingQuests(k, value)
			end,
		}
	end
    --]]
    
	Cartographer.options.args["Trainers"] = {
		name = "Trainers",
		desc = self.notes,
		type = 'group',
		args = aceopts,
		handler = self,
	}
	AceLibrary("AceConsole-2.0"):InjectAceOptionsTable(self, Cartographer.options.args["Trainers"])

    if (not self.db.faction.notes) then self.db.faction.notes = {} end
end

function mod:OnEnable()
	if Cartographer_Notes then
		if not self.iconsregistered then
			for k,v in pairs(self.icon) do
				Cartographer_Notes:RegisterIcon(k, v)
			end
			self.iconsregistered = true
		end

		Cartographer_Notes:RegisterNotesDatabase('Trainers', self.db.faction.notes, self)
	else
		Cartographer:ToggleModuleActive(self, false)
	end
    
    if (School) or (TrainerFu) then
        Cartographer:ToggleModuleActive(self, false)
    end
    
    if (Cartographer:IsModuleActive(self)) then
        self:RegisterEvent("TRAINER_SHOW", function() self:RegisterTrainer(UnitName("NPC")) end, 2)
    end
end

function mod:OnDisable()
	self:UnregisterAllEvents()

	if Cartographer_Notes then
		Cartographer_Notes:UnregisterNotesDatabase('Trainers')
	end
end

function mod:SetNote(trainer)
	local x, y = GetPlayerMapPosition("player")
	if x == 0 and y == 0 then return end
	local zone = GetRealZoneText()
    
    local note = {}
    note.icon = L["Trainer"]
    note.title = trainer.name
    note.info = L["Trainer"]
    
	for zone, x_p, y_p, icon, creator, data in Cartographer_Notes:IterateNearbyNotes(zone, x, y, 5, "Trainers", nil, true) do
		if data.title == note.title then
			Cartographer_Notes:DeleteNote(zone, x_p, y_p)
		end
	end
	Cartographer_Notes:SetNote(zone, x, y, note.icon, "Trainers", note)
end

function mod:RegisterTrainer(trainer)
    if (not trainer) then return end
    if (type(trainer) ~= "string") then return end
    
    if (not mod.trainers) then mod.trainers = {} end
    if (not mod.trainers[trainer]) then 
        mod.trainers[trainer] = {} 
        mod.trainers[trainer].name = trainer
    end
    
    self:SetNote(mod.trainers[trainer])
end

function mod:IsNoteHidden(zone, id, trainertype)
	return not self.db.profile.filter[trainertype]
end

function mod:IsShowingTrainers(trainertype)
	return self.db.profile.filter[trainertype]
end

function mod:ToggleShowingTrainers(trainertype, value)
	if value == nil then
		value = not self.db.profile.filter[trainertype]
	end
	self.db.profile.filter[trainertype] = value

	Cartographer_Notes:RefreshMap()
end
